clear all;clc;

global Multi_Class;
rand('seed',sum(100*clock));

%------------------------------------- load data set ----------------------------------------------

%------------------ train data information ----------------
train_data_all=load('train_data.txt');    % load the test data;
[sample_size,temp]=size(train_data_all);
Multi_Class=length(unique(train_data_all(:,1))); % get the number of classes

%------------------ test data information -----------------
test_data=load('test_data.txt');    % load the test data;
x_test=test_data;
[n_test,p]=size(x_test);

%--------------------- Tune part, using the leave one out method ----------------------------------%
EGKL_sum=zeros(1,17); %gamma range from 2^(-8) to 2^(8), 17 gamma candiate value%

%-------------------- Set the train data and tune data --------------------
temp=randperm(sample_size);
tune_sub=temp(1:floor(sample_size/2));
tune_sub=sort(tune_sub);    % randomly choose half of the train data as the train set and let the rest as the tune set
train_data = train_data_all;
train_data(tune_sub,:)=[]; % leave one out from the train data
train_simp = Mul2Sim (train_data , Multi_Class );
tune_data = train_data_all(tune_sub,:); % let the left be the tune data
tune_simp = Mul2Sim( tune_data, Multi_Class);

for i=1:Multi_Class-1
    for j=i+1:Multi_Class
        train_temp = train_simp{i,j};
        tune_temp = tune_simp{i,j};

        %----------------- Tunning part to get the best gamma ----------------------------------
        y_train=train_temp(:,1);
        x_train=train_temp;
        x_train(:,1)=[];

        [n_train,p]=size(x_train);

        %--------------- get K_train ---------------
        sigma=median(pdist(x_train));
        sigma=2*sigma^2;
        y=pdist(x_train,'euclidean');
        y=y.^2;

        temp=-y/sigma;
        K_train=squareform(temp);
        K_train=exp(K_train);

        y_tune=tune_temp(:,1);
        x_tune=tune_temp;
        x_tune(:,1)=[];

        [n_tune,p]=size(x_tune);
        %--------------- get K_tune ---------------
        X_tt=[x_train;x_tune];
        d_eu=pdist(X_tt);
        d_eu=-d_eu.^2/sigma;
        K_tt=squareform(d_eu);
        K_tt=exp(K_tt);
        K_tune=K_tt(n_train+1:n_train+n_tune,1:n_train);

        EGKL1=get_one_EGKL(K_train,y_train,K_tune,y_tune);
        EGKL_sum=EGKL_sum+EGKL1;
    end
end

[EGKLmax, gamma_i]=max(EGKL_sum);

gamma_EGKL=2^(gamma_i-9);

%----------------- Test part using the whole train data --------------%
for i=1:Multi_Class-1
    for j=i+1:Multi_Class

        train_temp = train_simp{i,j};
        y_train=train_temp(:,1);
        x_train=train_temp(:,2:end);
        [n_train,p]=size(x_train);

        %--------------- get K_train ---------------
        sigma=median(pdist(x_train));
        sigma=2*sigma^2;
        y=pdist(x_train,'euclidean');
        y=y.^2;

        temp=-y/sigma;
        K_train=squareform(temp);
        K_train=exp(K_train);

        %-------------- get K_test -------------------
        X_tt2=[x_train;x_test];
        d_eu2=pdist(X_tt2);
        d_eu2=-d_eu2.^2/sigma;
        K_tt2=zeros(n_train+n_test,n_train+n_test);
        K_tt2=squareform(d_eu2);
        K_tt2=exp(K_tt2);
        K_test=K_tt2(n_train+1:n_train+n_test,1:n_train);

        [pi,pj] = get_p_simp(K_test,K_train,y_train,gamma_EGKL);

        prob_EGKL{i,j} = [pi,pj];
    end
end

%--------------------------------- output part ---------------------------------------------------
prediction_result=prediction(prob_EGKL);
dlmwrite('test_data_prediction.txt',prediction_result,'-append')
